CHANGE_INFO = "change_info"
POST_MESSAGES = "post_messages"
EDIT_MESSAGES = "edit_messages"
DELETE_MESSAGES = "delete_messages"
RESTRICT_MEMBERS = "restrict_members"
INVITE_USERS = "invite_users"
PIN_MESSAGES = "pin_messages"
PROMOTE_MEMBERS = "promote_members"
MANAGE_VIDEO_CHATS = "manage_video_chats"
ANONYMOUS = "anonymous"
MANAGE_CHAT = "manage_chat"
